@@ -2,12 +2,12 @@
    +----------------------------------------------------------------------+
    | Zend Engine                                                          |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1998-2001 Zend Technologies Ltd. (http://www.zend.com) |
+   | Copyright (c) 1998-2003 Zend Technologies Ltd. (http://www.zend.com) |
    +----------------------------------------------------------------------+
-   | This source file is subject to version 0.92 of the Zend license,     |
-   | that is bundled with this package in the file LICENSE, and is        | 
+   | This source file is subject to version 2.00 of the Zend license,     |
+   | that is bundled with this package in the file LICENSE, and is        |
    | available at through the world-wide-web at                           |
-   | http://www.zend.com/license/0_92.txt.                                |
+   | http://www.zend.com/license/2_00.txt.                                |
    | If you did not receive a copy of the Zend license and are unable to  |
    | obtain it through the world-wide-web, please send a note to          |
    | license@zend.com so we can mail you a copy immediately.              |
@@ -25,6 +25,9 @@
 #include "zend_API.h"
 #include "zend_fast_cache.h"
 
+#ifdef ZEND_MULTIBYTE
+#include "zend_multibyte.h"
+#endif /* ZEND_MULTIBYTE */
 
 ZEND_API zend_op_array *(*zend_compile_file)(zend_file_handle *file_handle, int type TSRMLS_DC);
 
@@ -81,6 +84,15 @@ void zend_init_compiler_data_structures(TSRMLS_D)
 	CG(handle_op_arrays) = 1;
 	CG(in_compilation) = 0;
 	init_compiler_declarables(TSRMLS_C);
+	CG(zend_lineno) = 0;
+#ifdef ZEND_MULTIBYTE
+	CG(script_encoding_list) = NULL;
+	CG(script_encoding_list_size) = 0;
+	CG(internal_encoding) = NULL;
+	CG(encoding_detector) = NULL;
+	CG(encoding_converter) = NULL;
+	CG(multibyte_oddlen) = NULL;
+#endif /* ZEND_MULTIBYTE */
 }
 
 
@@ -105,6 +117,11 @@ void shutdown_compiler(TSRMLS_D)
 	zend_stack_destroy(&CG(list_stack));
 	zend_hash_destroy(&CG(filenames_table));
 	zend_llist_destroy(&CG(open_files));
+#ifdef ZEND_MULTIBYTE
+	if (CG(script_encoding_list)) {
+		efree(CG(script_encoding_list));
+	}
+#endif /* ZEND_MULTIBYTE */
 }
 
 
@@ -382,7 +399,7 @@ void zend_do_while_end(znode *while_token, znode *close_bracket_token TSRMLS_DC)
 	opline->op1.u.opline_num = while_token->u.opline_num;
 	SET_UNUSED(opline->op1);
 	SET_UNUSED(opline->op2);
-	
+
 	/* update while's conditional jmp */
 	CG(active_op_array)->opcodes[close_bracket_token->u.opline_num].op2.u.opline_num = get_next_op_number(CG(active_op_array));
 
@@ -492,7 +509,7 @@ void zend_do_if_after_statement(znode *closing_bracket_token, unsigned char init
 	}
 	zend_stack_top(&CG(bp_stack), (void **) &jmp_list_ptr);
 	zend_llist_add_element(jmp_list_ptr, &if_end_op_number);
-	
+
 	CG(active_op_array)->opcodes[closing_bracket_token->u.opline_num].op2.u.opline_num = if_end_op_number+1;
 	SET_UNUSED(opline->op1);
 	SET_UNUSED(opline->op2);
@@ -551,6 +568,9 @@ void zend_do_end_variable_parse(int type, int arg_offset TSRMLS_DC)
 				opline->opcode += 3;
 				break;
 			case BP_VAR_IS:
+				if (opline->opcode == ZEND_FETCH_DIM_W && opline->op2.op_type == IS_UNUSED) {
+					zend_error(E_COMPILE_ERROR, "Cannot use [] for reading");
+				}
 				opline->opcode += 6; /* 3+3 */
 				break;
 			case BP_VAR_FUNC_ARG:
@@ -666,7 +686,7 @@ void zend_do_add_variable(znode *result, znode *op1, znode *op2 TSRMLS_DC)
 	*result = opline->result;
 }
 
-	
+
 void zend_do_free(znode *op1 TSRMLS_DC)
 {
 	if (op1->op_type==IS_TMP_VAR) {
@@ -711,7 +731,7 @@ void zend_do_free(znode *op1 TSRMLS_DC)
 		}
 	} else if (op1->op_type == IS_CONST) {
 		zval_dtor(&op1->u.constant);
-	}		
+	}
 }
 
 
@@ -732,7 +752,17 @@ void zend_do_begin_function_declaration(znode *function_token, znode *function_n
 	op_array.return_reference = return_reference;
 
 	if (is_method) {
-		zend_hash_update(&CG(active_class_entry)->function_table, name, name_len+1, &op_array, sizeof(zend_op_array), (void **) &CG(active_op_array));
+		if (zend_hash_add(&CG(active_class_entry)->function_table, name, name_len+1, &op_array, sizeof(zend_op_array), (void **) &CG(active_op_array)) == FAILURE) {
+			zend_op_array *child_op_array, *parent_op_array;
+			if (CG(active_class_entry)->parent
+				&& (zend_hash_find(&CG(active_class_entry)->function_table, name, name_len+1, (void **) &child_op_array) == SUCCESS)
+				&& (zend_hash_find(&CG(active_class_entry)->parent->function_table, name, name_len+1, (void **) &parent_op_array) == SUCCESS)
+				&& (child_op_array->refcount == parent_op_array->refcount)) {
+				zend_hash_update(&CG(active_class_entry)->function_table, name, name_len+1, &op_array, sizeof(zend_op_array), (void **) &CG(active_op_array));
+			} else {
+				zend_error(E_COMPILE_ERROR, "Cannot redeclare %s()", name);
+			}
+		}
 	} else {
 		zend_op *opline = get_next_op(CG(active_op_array) TSRMLS_CC);
 
@@ -756,11 +786,11 @@ void zend_do_begin_function_declaration(znode *function_token, znode *function_n
 		SET_UNUSED(opline->op1);
 		SET_UNUSED(opline->op2);
 	}
-	
+
 	{
 		/* Push a seperator to the switch and foreach stacks */
 		zend_switch_entry switch_entry;
-	
+
 		switch_entry.cond.op_type = IS_UNUSED;
 		switch_entry.default_case = 0;
 		switch_entry.control_var = 0;
@@ -819,7 +849,7 @@ void zend_do_receive_arg(int op, znode *var, znode *offset, znode *initializatio
 int zend_do_begin_function_call(znode *function_name TSRMLS_DC)
 {
 	zend_function *function;
-	
+
 	zend_str_tolower(function_name->u.constant.value.str.val, function_name->u.constant.value.str.len);
 	if (zend_hash_find(CG(function_table), function_name->u.constant.value.str.val, function_name->u.constant.value.str.len+1, (void **) &function)==FAILURE) {
 		znode tmp = *function_name;
@@ -828,22 +858,22 @@ int zend_do_begin_function_call(znode *function_name TSRMLS_DC)
 		zend_do_begin_dynamic_function_call(&tmp TSRMLS_CC);
 		return 1; /* Dynamic */
 	}
-	
+
 	switch (function->type) {
 		case ZEND_USER_FUNCTION:	{
 				zend_op_array *op_array = (zend_op_array *) function;
-				
+
 				zend_stack_push(&CG(function_call_stack), (void *) &op_array, sizeof(zend_function *));
 			}
 			break;
 		case ZEND_INTERNAL_FUNCTION: {
 				zend_internal_function *internal_function = (zend_internal_function *) function;
-				
+
 				zend_stack_push(&CG(function_call_stack), (void *) &internal_function, sizeof(zend_function *));
 			}
 			break;
 	}
-	zend_do_extended_fcall_begin(TSRMLS_C); 
+	zend_do_extended_fcall_begin(TSRMLS_C);
 	return 0;
 }
 
@@ -874,7 +904,7 @@ void zend_do_begin_dynamic_function_call(znode *function_name TSRMLS_DC)
 		SET_UNUSED(opline->op1);
 	}
 	zend_stack_push(&CG(function_call_stack), (void *) &ptr, sizeof(zend_function *));
-	zend_do_extended_fcall_begin(TSRMLS_C); 
+	zend_do_extended_fcall_begin(TSRMLS_C);
 }
 
 
@@ -905,7 +935,7 @@ void zend_do_begin_class_member_function_call(znode *class_name, znode *function
 void zend_do_end_function_call(znode *function_name, znode *result, znode *argument_list, int is_method, int is_dynamic_fcall TSRMLS_DC)
 {
 	zend_op *opline = get_next_op(CG(active_op_array) TSRMLS_CC);
-	
+
 	if (function_name->op_type==IS_CONST && !is_method && !is_dynamic_fcall) {
 		opline->opcode = ZEND_DO_FCALL;
 	} else {
@@ -930,7 +960,7 @@ void zend_do_pass_param(znode *param, int op, int offset TSRMLS_DC)
 	zend_function **function_ptr_ptr, *function_ptr;
 	int send_by_reference;
 
-						
+
 	zend_stack_top(&CG(function_call_stack), (void **) &function_ptr_ptr);
 	function_ptr = *function_ptr_ptr;
 
@@ -942,8 +972,7 @@ void zend_do_pass_param(znode *param, int op, int offset TSRMLS_DC)
 					"If you would like to enable call-time pass-by-reference, you can set "
 					"allow_call_time_pass_reference to true in your INI file.  "
 					"However, future versions may not support this any longer. ",
-					(function_ptr?function_ptr->common.function_name:"[runtime function name]"),
-					offset+1);
+					(function_ptr?function_ptr->common.function_name:"[runtime function name]"));
 	}
 
 	if (function_ptr) {
@@ -1013,11 +1042,11 @@ void zend_do_pass_param(znode *param, int op, int offset TSRMLS_DC)
 static int generate_free_switch_expr(zend_switch_entry *switch_entry TSRMLS_DC)
 {
 	zend_op *opline;
-	
+
 	if (switch_entry->cond.op_type!=IS_VAR && switch_entry->cond.op_type!=IS_TMP_VAR) {
 		return 1;
 	}
-	
+
 	opline = get_next_op(CG(active_op_array) TSRMLS_CC);
 
 	opline->opcode = ZEND_SWITCH_FREE;
@@ -1030,7 +1059,7 @@ static int generate_free_switch_expr(zend_switch_entry *switch_entry TSRMLS_DC)
 static int generate_free_foreach_copy(znode *foreach_copy TSRMLS_DC)
 {
 	zend_op *opline;
-	
+
 	if (foreach_copy->op_type!=IS_VAR && foreach_copy->op_type!=IS_TMP_VAR) {
 		return 1;
 	}
@@ -1047,7 +1076,7 @@ static int generate_free_foreach_copy(znode *foreach_copy TSRMLS_DC)
 void zend_do_return(znode *expr, int do_end_vparse TSRMLS_DC)
 {
 	zend_op *opline;
-	
+
 	if (do_end_vparse) {
 		if (CG(active_op_array)->return_reference) {
 			zend_do_end_variable_parse(BP_VAR_W, 0 TSRMLS_CC);
@@ -1071,7 +1100,7 @@ void zend_do_return(znode *expr, int do_end_vparse TSRMLS_DC)
 	opline = get_next_op(CG(active_op_array) TSRMLS_CC);
 
 	opline->opcode = ZEND_RETURN;
-	
+
 	if (expr) {
 		opline->op1 = *expr;
 	} else {
@@ -1141,8 +1170,19 @@ ZEND_API int do_bind_function_or_class(zend_op *opline, HashTable *function_tabl
 
 				zend_hash_find(function_table, opline->op1.u.constant.value.str.val, opline->op1.u.constant.value.str.len, (void *) &function);
 				if (zend_hash_add(function_table, opline->op2.u.constant.value.str.val, opline->op2.u.constant.value.str.len+1, function, sizeof(zend_function), NULL)==FAILURE) {
+					zend_function *function;
+
 					if (!compile_time) {
-						zend_error(E_ERROR, "Cannot redeclare %s()", opline->op2.u.constant.value.str.val);
+						if (zend_hash_find(function_table, opline->op2.u.constant.value.str.val, opline->op2.u.constant.value.str.len+1, (void *) &function)==SUCCESS
+							&& function->type==ZEND_USER_FUNCTION
+							&& ((zend_op_array *) function)->last>0) {
+							zend_error(E_ERROR, "Cannot redeclare %s() (previously declared in %s:%d)",
+										opline->op2.u.constant.value.str.val,
+										((zend_op_array *) function)->filename,
+										((zend_op_array *) function)->opcodes[0].lineno);
+						} else {
+							zend_error(E_ERROR, "Cannot redeclare %s()", opline->op2.u.constant.value.str.val);
+						}
 					}
 					return FAILURE;
 				} else {
@@ -1177,7 +1217,7 @@ ZEND_API int do_bind_function_or_class(zend_op *opline, HashTable *function_tabl
 				char *class_name, *parent_name;
 				int found_ce;
 
-				
+
 				found_ce = zend_hash_find(class_table, opline->op1.u.constant.value.str.val, opline->op1.u.constant.value.str.len, (void **) &ce);
 
 				/* Restore base class / derived class names */
@@ -1271,7 +1311,7 @@ void zend_do_boolean_or_begin(znode *expr1, znode *op_token TSRMLS_DC)
 	}
 	opline->op1 = *expr1;
 	SET_UNUSED(opline->op2);
-	
+
 	op_token->u.opline_num = next_op_number;
 
 	*expr1 = opline->result;
@@ -1306,7 +1346,7 @@ void zend_do_boolean_and_begin(znode *expr1, znode *op_token TSRMLS_DC)
 	}
 	opline->op1 = *expr1;
 	SET_UNUSED(opline->op2);
-	
+
 	op_token->u.opline_num = next_op_number;
 
 	*expr1 = opline->result;
@@ -1399,7 +1439,7 @@ void zend_do_switch_end(znode *case_list TSRMLS_DC)
 {
 	zend_op *opline;
 	zend_switch_entry *switch_entry_ptr;
-	
+
 	zend_stack_top(&CG(switch_cond_stack), (void **) &switch_entry_ptr);
 
 	if (case_list->op_type != IS_UNUSED) { /* non-empty switch */
@@ -1458,7 +1498,7 @@ void zend_do_case_before_statement(znode *case_list, znode *case_token, znode *c
 		zval_copy_ctor(&opline->op1.u.constant);
 	}
 	result = opline->result;
-	
+
 	next_op_number = get_next_op_number(CG(active_op_array));
 	opline = get_next_op(CG(active_op_array) TSRMLS_CC);
 	opline->opcode = ZEND_JMPZ;
@@ -1544,7 +1584,7 @@ void zend_do_begin_class_declaration(znode *class_name, znode *parent_class_name
 	CG(class_entry).refcount = (int *) emalloc(sizeof(int));
 	*CG(class_entry).refcount = 1;
 	CG(class_entry).constants_updated = 0;
-	
+
 	zend_str_tolower(CG(class_entry).name, CG(class_entry).name_length);
 
 	zend_hash_init(&CG(class_entry).function_table, 10, NULL, ZEND_FUNCTION_DTOR, 0);
@@ -1613,7 +1653,7 @@ void zend_do_begin_class_declaration(znode *class_name, znode *parent_class_name
 		opline->op2.u.constant.value.str.len = CG(class_entry).name_length;
 		opline->extended_value = ZEND_DECLARE_CLASS;
 	}
-	
+
 	zend_hash_update(CG(class_table), opline->op1.u.constant.value.str.val, opline->op1.u.constant.value.str.len, &CG(class_entry), sizeof(zend_class_entry), (void **) &CG(active_class_entry));
 }
 
@@ -1631,14 +1671,18 @@ void zend_do_end_class_declaration(TSRMLS_D)
 
 void zend_do_declare_property(znode *var_name, znode *value TSRMLS_DC)
 {
+	zval *property;
+
+	ALLOC_ZVAL(property);
+
 	if (value) {
-		zval *property;
-
-		ALLOC_ZVAL(property);
-
 		*property = value->u.constant;
-		zend_hash_update(&CG(active_class_entry)->default_properties, var_name->u.constant.value.str.val, var_name->u.constant.value.str.len+1, &property, sizeof(zval *), NULL);
+	} else {
+		INIT_PZVAL(property);
+		property->type = IS_NULL;
 	}
+
+	zend_hash_update(&CG(active_class_entry)->default_properties, var_name->u.constant.value.str.val, var_name->u.constant.value.str.len+1, &property, sizeof(zval *), NULL);
 	FREE_PNODE(var_name);
 }
 
@@ -1648,6 +1692,10 @@ void zend_do_fetch_property(znode *result, znode *object, znode *property TSRMLS
 	zend_op opline;
 	zend_llist *fetch_list_ptr;
 
+	if (property->op_type == IS_CONST
+		&& property->u.constant.type != IS_STRING) {
+		zend_error(E_COMPILE_ERROR, "Property name must be a string");
+	}
 	init_op(&opline TSRMLS_CC);
 	opline.opcode = ZEND_FETCH_OBJ_W;	/* the backpatching routine assumes W */
 	opline.result.op_type = IS_VAR;
@@ -1682,7 +1730,7 @@ void zend_do_begin_new_object(znode *new_token, znode *class_name TSRMLS_DC)
 {
 	zend_op *opline = get_next_op(CG(active_op_array) TSRMLS_CC);
 	unsigned char *ptr = NULL;
-	
+
 	opline->opcode = ZEND_NEW;
 	opline->result.op_type = IS_VAR;
 	opline->result.u.var = get_temporary_variable(CG(active_op_array));
@@ -1731,7 +1779,7 @@ void zend_do_fetch_constant(znode *result, znode *constant_name, int mode TSRMLS
 			break;
 		case ZEND_RT: {
 				zend_op *opline = get_next_op(CG(active_op_array) TSRMLS_CC);
-	
+
 				opline->opcode = ZEND_FETCH_CONSTANT;
 				opline->result.op_type = IS_TMP_VAR;
 				opline->result.u.var = get_temporary_variable(CG(active_op_array));
@@ -2012,9 +2060,6 @@ void zend_do_include_or_eval(int type, znode *result, znode *op1 TSRMLS_DC)
 		SET_UNUSED(opline->op2);
 		opline->op2.u.constant.value.lval = type;
 		*result = opline->result;
-		if (type==ZEND_REQUIRE) {
-			opline->result.u.EA.type |= EXT_TYPE_UNUSED;
-		}
 	}
 	zend_do_extended_fcall_end(TSRMLS_C);
 }
@@ -2102,9 +2147,9 @@ void zend_do_foreach_begin(znode *foreach_token, znode *array, znode *open_brack
 }
 
 
-void zend_do_foreach_cont(znode *value, znode *key, znode *as_token TSRMLS_DC)
+void zend_do_foreach_cont(znode *value, znode *key, znode *as_token, znode *foreach_token TSRMLS_DC)
 {
-	zend_op *opline = get_next_op(CG(active_op_array) TSRMLS_CC);
+	zend_op *opline;
 	znode result_value, result_key, dummy;
 
 	if (key->op_type != IS_UNUSED) {
@@ -2114,20 +2159,24 @@ void zend_do_foreach_cont(znode *value, znode *key, znode *as_token TSRMLS_DC)
 		tmp = key;
 		key = value;
 		value = tmp;
+
+		/* Mark extended_value in case both key and value are being used */
+		CG(active_op_array)->opcodes[foreach_token->u.opline_num].extended_value |= ZEND_FE_FETCH_WITH_KEY;
 	}
 
-	opline->opcode = ZEND_FETCH_DIM_TMP_VAR;
-	opline->result.op_type = IS_VAR;
-	opline->result.u.EA.type = 0;
-	opline->result.u.opline_num = get_temporary_variable(CG(active_op_array));
-	opline->op1 = *as_token;
-	opline->op2.op_type = IS_CONST;
-	opline->op2.u.constant.type = IS_LONG;
-	opline->op2.u.constant.value.lval = 0;
-	opline->extended_value = ZEND_FETCH_STANDARD; /* ignored in fetch_dim_tmp_var, but what the hell. */
-	result_value = opline->result;
-
 	if (key->op_type != IS_UNUSED) {
+		opline = get_next_op(CG(active_op_array) TSRMLS_CC);
+		opline->opcode = ZEND_FETCH_DIM_TMP_VAR;
+		opline->result.op_type = IS_VAR;
+		opline->result.u.EA.type = 0;
+		opline->result.u.opline_num = get_temporary_variable(CG(active_op_array));
+		opline->op1 = *as_token;
+		opline->op2.op_type = IS_CONST;
+		opline->op2.u.constant.type = IS_LONG;
+		opline->op2.u.constant.value.lval = 0;
+		opline->extended_value = ZEND_FETCH_STANDARD; /* ignored in fetch_dim_tmp_var, but what the hell. */
+		result_value = opline->result;
+
 		opline = get_next_op(CG(active_op_array) TSRMLS_CC);
 		opline->opcode = ZEND_FETCH_DIM_TMP_VAR;
 		opline->result.op_type = IS_VAR;
@@ -2139,15 +2188,17 @@ void zend_do_foreach_cont(znode *value, znode *key, znode *as_token TSRMLS_DC)
 		opline->op2.u.constant.value.lval = 1;
 		opline->extended_value = ZEND_FETCH_STANDARD; /* ignored in fetch_dim_tmp_var, but what the hell. */
 		result_key = opline->result;
+	} else {
+		result_value = CG(active_op_array)->opcodes[foreach_token->u.opline_num].result;
 	}
 
 	zend_do_assign(&dummy, value, &result_value TSRMLS_CC);
 	CG(active_op_array)->opcodes[CG(active_op_array)->last-1].result.u.EA.type |= EXT_TYPE_UNUSED;
 	if (key->op_type != IS_UNUSED) {
 		zend_do_assign(&dummy, key, &result_key TSRMLS_CC);
-		CG(active_op_array)->opcodes[CG(active_op_array)->last-1].result.u.EA.type |= EXT_TYPE_UNUSED;	
+		CG(active_op_array)->opcodes[CG(active_op_array)->last-1].result.u.EA.type |= EXT_TYPE_UNUSED;
+		zend_do_free(as_token TSRMLS_CC);
 	}
-	zend_do_free(as_token TSRMLS_CC);
 
 	do_begin_loop(TSRMLS_C);
 	INC_BPC(CG(active_op_array));
@@ -2188,17 +2239,45 @@ void zend_do_declare_stmt(znode *var, znode *val TSRMLS_DC)
 	if (!zend_binary_strcasecmp(var->u.constant.value.str.val, var->u.constant.value.str.len, "ticks", sizeof("ticks")-1)) {
 		convert_to_long(&val->u.constant);
 		CG(declarables).ticks = val->u.constant;
+#ifdef ZEND_MULTIBYTE
+	} else if (!zend_binary_strcasecmp(var->u.constant.value.str.val, var->u.constant.value.str.len, "encoding", sizeof("encoding")-1)) {
+		zend_encoding *new_encoding, *old_encoding;
+		zend_multibyte_filter old_input_filter;
+
+		if (val->u.constant.type == IS_CONSTANT) {
+			zend_error(E_COMPILE_ERROR, "Cannot use constants as encoding");
+		}
+		convert_to_string(&val->u.constant);
+		new_encoding = zend_multibyte_fetch_encoding(val->u.constant.value.str.val);
+		if (!new_encoding) {
+			zend_error(E_COMPILE_WARNING, "Unsupported encoding [%s]", val->u.constant.value.str.val);
+		} else {
+			old_input_filter = LANG_SCNG(input_filter);
+			old_encoding = LANG_SCNG(script_encoding);
+			zend_multibyte_set_filter(new_encoding TSRMLS_CC);
+
+			if (old_input_filter != LANG_SCNG(input_filter) ||
+				((old_input_filter == zend_multibyte_encoding_filter) &&
+				 (new_encoding != old_encoding))) {
+				zend_yyinput_again(old_input_filter, old_encoding TSRMLS_CC);
+			}
+		}
+		efree(val->u.constant.value.str.val);
+#endif /* ZEND_MULTIBYTE */
 	}
 	zval_dtor(&var->u.constant);
 }
 
 
-void zend_do_declare_end(TSRMLS_D)
+void zend_do_declare_end(znode *declare_token TSRMLS_DC)
 {
 	zend_declarables *declarables;
 
 	zend_stack_top(&CG(declare_stack), (void **) &declarables);
-	CG(declarables) = *declarables;
+	/* We should restore if there was more than (current - start) - (ticks?1:0) opcodes */
+	if ((get_next_op_number(CG(active_op_array)) - declare_token->u.opline_num) - ((CG(declarables).ticks.value.lval)?1:0)) {
+		CG(declarables) = *declarables;
+	}
 }
 
 
@@ -2216,7 +2295,7 @@ void zend_do_end_heredoc(TSRMLS_D)
 		if (opline->op2.u.constant.value.str.val[opline->op2.u.constant.value.str.len-1]=='\r') {
 			opline->op2.u.constant.value.str.val[(opline->op2.u.constant.value.str.len--)-1] = 0;
 		}
-	}	
+	}
 }
 
 
@@ -2261,7 +2340,7 @@ void zend_do_begin_qm_op(znode *cond, znode *qm_token TSRMLS_DC)
 {
 	int jmpz_op_number = get_next_op_number(CG(active_op_array));
 	zend_op *opline;
-	
+
 	opline = get_next_op(CG(active_op_array) TSRMLS_CC);
 
 	opline->opcode = ZEND_JMPZ;
@@ -2304,7 +2383,7 @@ void zend_do_qm_false(znode *result, znode *false_value, znode *qm_token, znode
 	opline->result = *qm_token;
 	opline->op1 = *false_value;
 	SET_UNUSED(opline->op2);
-	
+
 	CG(active_op_array)->opcodes[colon_token->u.opline_num].op1.u.opline_num = get_next_op_number(CG(active_op_array));
 
 	*result = opline->result;
@@ -2316,11 +2395,11 @@ void zend_do_qm_false(znode *result, znode *false_value, znode *qm_token, znode
 void zend_do_extended_info(TSRMLS_D)
 {
 	zend_op *opline;
-	
+
 	if (!CG(extended_info)) {
 		return;
 	}
-	
+
 	opline = get_next_op(CG(active_op_array) TSRMLS_CC);
 
 	opline->opcode = ZEND_EXT_STMT;
@@ -2332,11 +2411,11 @@ void zend_do_extended_info(TSRMLS_D)
 void zend_do_extended_fcall_begin(TSRMLS_D)
 {
 	zend_op *opline;
-	
+
 	if (!CG(extended_info)) {
 		return;
 	}
-	
+
 	opline = get_next_op(CG(active_op_array) TSRMLS_CC);
 
 	opline->opcode = ZEND_EXT_FCALL_BEGIN;
@@ -2348,11 +2427,11 @@ void zend_do_extended_fcall_begin(TSRMLS_D)
 void zend_do_extended_fcall_end(TSRMLS_D)
 {
 	zend_op *opline;
-	
+
 	if (!CG(extended_info)) {
 		return;
 	}
-	
+
 	opline = get_next_op(CG(active_op_array) TSRMLS_CC);
 
 	opline->opcode = ZEND_EXT_FCALL_END;
@@ -2373,7 +2452,6 @@ void zend_do_ticks(TSRMLS_D)
 	}
 }
 
-
 int zend_register_auto_global(char *name, uint name_len TSRMLS_DC)
 {
 	return zend_hash_add_empty_element(CG(auto_globals), name, name_len+1);
@@ -2384,6 +2462,7 @@ int zendlex(znode *zendlval TSRMLS_DC)
 {
 	int retval;
 
+again:
 	if (CG(increment_lineno)) {
 		CG(zend_lineno)++;
 		CG(increment_lineno) = 0;
@@ -2395,8 +2474,8 @@ int zendlex(znode *zendlval TSRMLS_DC)
 		case T_COMMENT:
 		case T_OPEN_TAG:
 		case T_WHITESPACE:
-			retval = zendlex(zendlval TSRMLS_CC);
-			break;
+			goto again;
+
 		case T_CLOSE_TAG:
 			if (LANG_SCNG(yy_text)[LANG_SCNG(yy_leng)-1]=='\n'
 				|| (LANG_SCNG(yy_text)[LANG_SCNG(yy_leng)-2]=='\r' && LANG_SCNG(yy_text)[LANG_SCNG(yy_leng)-1])) {
@@ -2411,7 +2490,7 @@ int zendlex(znode *zendlval TSRMLS_DC)
 			efree(zendlval->u.constant.value.str.val);
 			break;
 	}
-		
+
 	INIT_PZVAL(&zendlval->u.constant);
 	zendlval->op_type = IS_CONST;
 	return retval;
