@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP version 4.0                                                      |
+   | PHP Version 4                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2001 The PHP Group                                |
+   | Copyright (c) 1997-2003 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -12,7 +12,7 @@
    | obtain it through the world-wide-web, please send a note to          |
    | license@php.net so we can mail you a copy immediately.               |
    +----------------------------------------------------------------------+
-   | Authors: Stanislav Malyshev <stas@php.net>                           |
+   | Author: Stanislav Malyshev <stas@php.net>                            |
    +----------------------------------------------------------------------+
  */
 
@@ -28,9 +28,11 @@
 #if HAVE_GMP
 
 #include <gmp.h>
-/* If you declare any globals in php_gmp.h uncomment this:
-ZEND_DECLARE_MODULE_GLOBALS(gmp)
-*/
+
+/* Needed for gmp_random() */
+#include "ext/standard/php_rand.h"
+#include "ext/standard/php_lcg.h"
+#define GMP_ABS(x) ((x) >= 0 ? (x) : -(x))
 
 /* True global resources - no need for thread safety here */
 static int le_gmp;
@@ -86,19 +88,21 @@ function_entry gmp_functions[] = {
 /* {{{ gmp_module_entry
  */
 zend_module_entry gmp_module_entry = {
-    STANDARD_MODULE_HEADER,
+	STANDARD_MODULE_HEADER,
 	"gmp",
 	gmp_functions,
 	ZEND_MODULE_STARTUP_N(gmp),
 	ZEND_MODULE_SHUTDOWN_N(gmp),
-	NULL,		/* Replace with NULL if there's nothing to do at request start */
-	NULL,	    /* Replace with NULL if there's nothing to do at request end */
+	NULL,
+	ZEND_MODULE_DEACTIVATE_N(gmp),
 	ZEND_MODULE_INFO_N(gmp),
-    NO_VERSION_YET,
+	NO_VERSION_YET,
 	STANDARD_MODULE_PROPERTIES
 };
 /* }}} */
 
+ZEND_DECLARE_MODULE_GLOBALS(gmp)
+
 #ifdef COMPILE_DL_GMP
 ZEND_GET_MODULE(gmp)
 #endif
@@ -135,10 +139,20 @@ static void gmp_efree(void *ptr, size_t size)
 }
 /* }}} */
 
+/* {{{ php_gmp_init_globals
+ */
+static void php_gmp_init_globals(zend_gmp_globals *gmp_globals)
+{
+	gmp_globals->rand_initialized = 0;
+}
+/* }}} */
+
 /* {{{ ZEND_MINIT_FUNCTION
  */
 ZEND_MODULE_STARTUP_D(gmp)
 {
+    ZEND_INIT_MODULE_GLOBALS(gmp, php_gmp_init_globals, NULL);
+
 	le_gmp = zend_register_list_destructors_ex(_php_gmpnum_free, NULL, GMP_RESOURCE_NAME, module_number);
 	REGISTER_LONG_CONSTANT("GMP_ROUND_ZERO", GMP_ROUND_ZERO, CONST_CS | CONST_PERSISTENT);
 	REGISTER_LONG_CONSTANT("GMP_ROUND_PLUSINF", GMP_ROUND_PLUSINF, CONST_CS | CONST_PERSISTENT);
@@ -150,6 +164,19 @@ ZEND_MODULE_STARTUP_D(gmp)
 }
 /* }}} */
 
+/* {{{ ZEND_RSHUTDOWN_FUNCTION
+ */
+ZEND_MODULE_DEACTIVATE_D(gmp)
+{
+	if (GMPG(rand_initialized)) {
+		gmp_randclear(GMPG(rand_state));
+		GMPG(rand_initialized) = 0;
+	}
+
+	return SUCCESS;
+}
+/* }}} */
+
 /* {{{ ZEND_MSHUTDOWN_FUNCTION
  */
 ZEND_MODULE_SHUTDOWN_D(gmp)
@@ -163,7 +190,7 @@ ZEND_MODULE_SHUTDOWN_D(gmp)
 ZEND_MODULE_INFO_D(gmp)
 {
 	php_info_print_table_start();
-	php_info_print_table_header(2, "gmp support", "enabled");
+	php_info_print_table_row(2, "gmp support", "enabled");
 	php_info_print_table_end();
 
 	/* Remove comments if you have entries in php.ini
@@ -193,6 +220,7 @@ if(Z_TYPE_PP(zval) == IS_RESOURCE) { \
 static int convert_to_gmp(mpz_t * *gmpnumber, zval **val, int base) 
 {
 	int ret = 0;
+	int skip_lead = 0;
 
 	*gmpnumber = emalloc(sizeof(mpz_t));
 	switch(Z_TYPE_PP(val)) {
@@ -207,14 +235,19 @@ static int convert_to_gmp(mpz_t * *gmpnumber, zval **val, int base)
 	case IS_STRING:
 		{
 			char *numstr = Z_STRVAL_PP(val);
-			if (base==0) {
-				if(numstr[0] == '0' && (numstr[1] == 'x' || numstr[1] == 'X')) {
-					base=16;
-				} else {
-					base=10;
+
+			if (Z_STRLEN_PP(val) > 2) {
+				if (numstr[0] == '0') {
+					if (numstr[1] == 'x' || numstr[1] == 'X') {
+						base = 16;
+						skip_lead = 1;
+					} else if (base != 16 && (numstr[1] == 'b' || numstr[1] == 'B')) {
+						base = 2;
+						skip_lead = 1;
+					}
 				}
 			}
-			ret = mpz_init_set_str(**gmpnumber, numstr, base);
+			ret = mpz_init_set_str(**gmpnumber, (skip_lead ? &numstr[2] : numstr), base);
 		}
 		break;
 	default:
@@ -222,8 +255,13 @@ static int convert_to_gmp(mpz_t * *gmpnumber, zval **val, int base)
 		efree(*gmpnumber);
 		return FAILURE;
 	}
+
+	if (ret) {
+		FREE_GMP_NUM(*gmpnumber);
+		return FAILURE;
+	}
 	
-	return ret?FAILURE:SUCCESS;
+	return SUCCESS;
 }
 /* }}} */
 
@@ -271,6 +309,10 @@ static inline void gmp_zval_binary_ui_op_ex(zval *return_value, zval **a_arg, zv
 		FETCH_GMP_ZVAL(gmpnum_b, b_arg);
 	}
 
+	if (!Z_LVAL_PP(b_arg)) {
+		RETURN_FALSE;
+	}
+
 	INIT_GMP_NUM(gmpnum_result);
 	if(use_ui && gmp_ui_op) {
 		if(allow_ui_return) {
@@ -300,7 +342,7 @@ static inline void gmp_zval_binary_ui_op2_ex(zval *return_value, zval **a_arg, z
 	mpz_t *gmpnum_a, *gmpnum_b, *gmpnum_result1, *gmpnum_result2;
 	zval r;
 	int use_ui=0;
-	unsigned long long_result;
+	unsigned long long_result = 0;
 
 	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
 	if(gmp_ui_op && Z_TYPE_PP(b_arg) == IS_LONG && Z_LVAL_PP(b_arg) >= 0) {
@@ -310,6 +352,10 @@ static inline void gmp_zval_binary_ui_op2_ex(zval *return_value, zval **a_arg, z
 		FETCH_GMP_ZVAL(gmpnum_b, b_arg);
 	}
 
+	if (!Z_LVAL_PP(b_arg)) {
+		RETURN_FALSE;
+	}
+
 	INIT_GMP_NUM(gmpnum_result1);
 	INIT_GMP_NUM(gmpnum_result2);
 
@@ -530,7 +576,19 @@ ZEND_FUNCTION(gmp_strval)
 		num_len++;
 	}
 	mpz_get_str(out_string, base, *gmpnum);
-	out_string[num_len] = '\0';
+
+	/* 
+	From GMP documentation for mpz_sizeinbase():
+	The returned value will be exact or 1 too big.  If base is a power of
+	2, the returned value will always be exact.
+
+	So let's check to see if we already have a \0 byte...
+	*/
+
+	if (out_string[num_len-1] == '\0')
+		num_len--;
+	else
+		out_string[num_len] = '\0';
 
 	RETVAL_STRINGL(out_string, num_len, 0);
 }
@@ -772,6 +830,10 @@ ZEND_FUNCTION(gmp_powm)
 	}
 	FETCH_GMP_ZVAL(gmpnum_mod, mod_arg);
 
+	if (!mpz_cmp_ui(*gmpnum_mod, 0)) {
+		RETURN_FALSE;
+	}
+
 	INIT_GMP_NUM(gmpnum_result);
 	if(use_ui) {
 		mpz_powm_ui(*gmpnum_result, *gmpnum_base, (unsigned long)Z_LVAL_PP(exp_arg), *gmpnum_mod);
@@ -793,7 +855,7 @@ ZEND_FUNCTION(gmp_sqrt)
 /* }}} */
 
 /* {{{ proto array gmp_sqrtrem(resource a)
-   Takes integer part of square root of a */
+   Square root with remainder */
 ZEND_FUNCTION(gmp_sqrtrem)
 {
 	zval **a_arg;
@@ -892,7 +954,7 @@ ZEND_FUNCTION(gmp_gcdext)
 	}
 
 	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
-	FETCH_GMP_ZVAL(gmpnum_b, a_arg);
+	FETCH_GMP_ZVAL(gmpnum_b, b_arg);
 
 	INIT_GMP_NUM(gmpnum_g);
 	INIT_GMP_NUM(gmpnum_s);
@@ -923,7 +985,7 @@ ZEND_FUNCTION(gmp_invert)
 	}
 
 	FETCH_GMP_ZVAL(gmpnum_a, a_arg);
-	FETCH_GMP_ZVAL(gmpnum_b, a_arg);
+	FETCH_GMP_ZVAL(gmpnum_b, b_arg);
 
 	INIT_GMP_NUM(gmpnum_result);
 	if(mpz_invert(*gmpnum_result, *gmpnum_a, *gmpnum_b)) {
@@ -1008,19 +1070,27 @@ ZEND_FUNCTION(gmp_random)
 
 	argc = ZEND_NUM_ARGS();
 
-	if (argc < 0  || argc > 1 || zend_get_parameters_ex(1, &limiter_arg) == FAILURE){
-		WRONG_PARAM_COUNT;
-	}
-
-	if(argc) {
+	if (argc == 0) {
+		limiter = 20;
+	} else if (argc == 1 && zend_get_parameters_ex(1, &limiter_arg) == SUCCESS) {
 		convert_to_long_ex(limiter_arg);
 		limiter = Z_LVAL_PP(limiter_arg);
 	} else {
-		limiter = 20;
+		WRONG_PARAM_COUNT;
 	}
 
 	INIT_GMP_NUM(gmpnum_result);
-	mpz_random(*gmpnum_result, limiter);
+
+	if (!GMPG(rand_initialized)) {
+		/* Initialize */
+		gmp_randinit_lc_2exp_size(GMPG(rand_state), 32L);
+
+		/* Seed */
+		gmp_randseed_ui(GMPG(rand_state), GENERATE_SEED());
+
+		GMPG(rand_initialized) = 1;
+	}
+	mpz_urandomb(*gmpnum_result, GMPG(rand_state), GMP_ABS (limiter) * __GMP_BITS_PER_MP_LIMB);
 
 	ZEND_REGISTER_RESOURCE(return_value, gmpnum_result, le_gmp);
 }
@@ -1084,12 +1154,12 @@ ZEND_FUNCTION(gmp_xor)
    Sets or clear bit in a */
 ZEND_FUNCTION(gmp_setbit)
 {
-	zval **a_arg, **ind_arg, **sc_arg;
+	zval **a_arg, **ind_arg, **set_c_arg;
 	int argc, index, set=1;
 	mpz_t *gmpnum_a;
 
 	argc = ZEND_NUM_ARGS();
-	if (argc < 2 || argc > 3 || zend_get_parameters_ex(argc, &a_arg, &ind_arg, &sc_arg) == FAILURE){
+	if (argc < 2 || argc > 3 || zend_get_parameters_ex(argc, &a_arg, &ind_arg, &set_c_arg) == FAILURE){
 		WRONG_PARAM_COUNT;
 	}
 
@@ -1100,8 +1170,8 @@ ZEND_FUNCTION(gmp_setbit)
 
 	switch (argc) {
 		case 3:
-			convert_to_long_ex(sc_arg);
-			set = Z_LVAL_PP(sc_arg);
+			convert_to_long_ex(set_c_arg);
+			set = Z_LVAL_PP(set_c_arg);
 			break;
 		case 2:
 			set = 1;
@@ -1161,7 +1231,7 @@ ZEND_FUNCTION(gmp_hamdist)
 	zval **a_arg, **b_arg;
 	mpz_t *gmpnum_a, *gmpnum_b;
 
-	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(1, &a_arg, &b_arg) == FAILURE){
+	if (ZEND_NUM_ARGS() != 2 || zend_get_parameters_ex(2, &a_arg, &b_arg) == FAILURE){
 		WRONG_PARAM_COUNT;
 	}
 	
@@ -1225,6 +1295,6 @@ static void _php_gmpnum_free(zend_rsrc_list_entry *rsrc TSRMLS_DC)
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: sw=4 ts=4 tw=78 fdm=marker
- * vim<600: sw=4 ts=4 tw=78
+ * vim600: sw=4 ts=4 fdm=marker
+ * vim<600: sw=4 ts=4
  */
