@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: mcve.c,v 1.14.2.11.2.1 2005/07/27 10:32:15 hyanantha Exp $ */
+/* $Id: mcve.c,v 1.14.2.10 2004/12/08 03:11:31 bradmssw Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -27,8 +27,10 @@
 
 #if PHP_WIN32
 #include "config.w32.h"
+#elif defined NETWARE
+#include "config.nw.h"
 #else
-#include <php_config.h>
+#include "php_config.h"
 #endif
 
 #if HAVE_MCVE
