@@ -20,7 +20,7 @@
   +----------------------------------------------------------------------+
 */
 
-/* $Id: snmp.c,v 1.70.2.13 2003/10/17 02:21:37 iliaa Exp $ */
+/* $Id: snmp.c,v 1.70.2.15 2004/12/07 00:34:33 iliaa Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -573,7 +573,7 @@ static void php_snmp(INTERNAL_FUNCTION_PARAMETERS, int st)
 	}
 
 	snmp_sess_init(&session);
-	strcpy (hostname, Z_STRVAL_PP(a1));
+	strlcpy(hostname, Z_STRVAL_PP(a1), sizeof(hostname));
 	if ((pptr = strchr (hostname, ':'))) {
 		remote_port = strtol (pptr + 1, NULL, 0);
 	}
@@ -769,7 +769,21 @@ static int netsnmp_session_set_sec_protocol(struct snmp_session *s, char *prot T
 			s->securityPrivProtoLen = OIDSIZE(usmDESPrivProtocol);
 			return (0);
 #ifdef HAVE_AES
-		} else if (!strcasecmp(prot, "AES128")) {
+		} else if (!strcasecmp(prot, "AES128")
+#ifdef SNMP_VALIDATE_ERR
+/* 
+* In Net-SNMP before 5.2, the following symbols exist:
+* usmAES128PrivProtocol, usmAES192PrivProtocol, usmAES256PrivProtocol
+* In an effort to be more standards-compliant, 5.2 removed the last two.
+* As of 5.2, the symbols are:
+* usmAESPrivProtocol, usmAES128PrivProtocol
+* 
+* As we want this extension to compile on both versions, we use the latter
+* symbol on purpose, as it's defined to be the same as the former.
+*/
+			|| !strcasecmp(prot, "AES")) {
+#else			
+		) {
 			s->securityPrivProto = usmAES128PrivProtocol;
 			s->securityPrivProtoLen = OIDSIZE(usmAES128PrivProtocol);
 			return (0);
@@ -781,6 +795,7 @@ static int netsnmp_session_set_sec_protocol(struct snmp_session *s, char *prot T
 			s->securityPrivProto = usmAES256PrivProtocol;
 			s->securityPrivProtoLen = OIDSIZE(usmAES256PrivProtocol);
 			return (0);
+#endif
 #endif
 		} else if (strlen(prot)) {
 			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid privacy protocol: %s", prot);
@@ -889,7 +904,7 @@ static void php_snmpv3(INTERNAL_FUNCTION_PARAMETERS, int st)
 
 	/* Reading the hostname and its optional non-default port number */
 	convert_to_string_ex(a1);
-	strcpy(hostname, Z_STRVAL_PP(a1));
+	strlcpy(hostname, Z_STRVAL_PP(a1), sizeof(hostname));
 	if ((pptr = strchr (hostname, ':'))) {
 		remote_port = strtol (pptr + 1, NULL, 0);
 	}
