@@ -19,7 +19,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: sockets.c,v 1.125.2.19 2003/08/28 20:01:30 iliaa Exp $ */
+/* $Id: sockets.c,v 1.125.2.24 2004/06/07 04:42:40 pollita Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -294,6 +294,7 @@ static int php_read(int bsd_socket, void *buf, size_t maxlen, int flags)
 
 	set_errno(0);
 
+	*t = '\0';
 	while (*t != '\n' && *t != '\r' && n < maxlen) {
 		if (m > 0) {
 			t++;
@@ -349,7 +350,7 @@ static char *php_strerror(int error TSRMLS_DC)
 		buf = hstrerror(error);
 #else
 		{
-			sprintf(SOCKETS_G(strerror_buf), "Host lookup error %d", error);
+			spprintf(&(SOCKETS_G(strerror_buf)), 0, "Host lookup error %d", error);
 			buf = SOCKETS_G(strerror_buf);
 		}
 #endif
@@ -363,7 +364,7 @@ static char *php_strerror(int error TSRMLS_DC)
 
 		if (FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM |	FORMAT_MESSAGE_IGNORE_INSERTS,
 				  NULL, error, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), (LPTSTR) &tmp, 0, NULL)) {
-			strlcpy(SOCKETS_G(strerror_buf), (char *) tmp, 10000);
+			SOCKETS_G(strerror_buf) = estrdup(tmp);
 			LocalFree(tmp);
 		
 			buf = SOCKETS_G(strerror_buf);
@@ -482,18 +483,18 @@ PHP_MINFO_FUNCTION(sockets)
 /* {{{ PHP_RINIT_FUNCTION */
 PHP_RINIT_FUNCTION(sockets)
 {
-	if ((SOCKETS_G(strerror_buf) = emalloc(16384))) 
-		return SUCCESS;
-	
-	return FAILURE;
+	return SUCCESS;
 }
 /* }}} */
 
 /* {{{ PHP_RSHUTDOWN_FUNCTION */
 PHP_RSHUTDOWN_FUNCTION(sockets)
 {
-	efree(SOCKETS_G(strerror_buf));
-	
+	if (SOCKETS_G(strerror_buf)) {
+		efree(SOCKETS_G(strerror_buf));
+		SOCKETS_G(strerror_buf) = NULL;
+	}
+
 	return SUCCESS;
 }
 /* }}} */
@@ -808,7 +809,10 @@ PHP_FUNCTION(socket_read)
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "rl|l", &arg1, &length, &type) == FAILURE)
 		return;
 
-	if(length<0) RETURN_FALSE;
+	/* overflow check */
+	if((length + 1) < 2) {
+		RETURN_FALSE;
+	}
 	tmpbuf = emalloc(length + 1);
 	
 	ZEND_FETCH_RESOURCE(php_sock, php_socket *, &arg1, -1, le_socket_name, le_socket);
@@ -1372,6 +1376,11 @@ PHP_FUNCTION(socket_recv)
 
 	ZEND_FETCH_RESOURCE(php_sock, php_socket *, &php_sock_res, -1, le_socket_name, le_socket);
 
+	/* overflow check */
+	if ((len + 1) < 2) {
+		RETURN_FALSE;
+	}
+
 	recv_buf = emalloc(len + 1);
 	memset(recv_buf, 0, len + 1);
 
@@ -1446,6 +1455,11 @@ PHP_FUNCTION(socket_recvfrom)
 
 	if(arg3<0) RETURN_FALSE;
 
+	/* overflow check */
+	if ((arg3 + 2) < 3) {
+		RETURN_FALSE;
+	}
+
 	recv_buf = emalloc(arg3 + 2);
 	memset(recv_buf, 0, arg3 + 2);
 	
