@@ -17,11 +17,10 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: credits.c,v 1.21.2.6.2.2 2005/09/20 22:20:30 johannes Exp $ */
+/* $Id: credits.c,v 1.21.2.6 2004/08/09 21:52:45 andi Exp $ */
 
 #include "php.h"
 #include "info.h"
-#include "SAPI.h"
 
 #define CREDIT_LINE(module, authors) php_info_print_table_row(2, module, authors)
 
@@ -31,15 +30,11 @@ PHPAPI void php_print_credits(int flag)
 {
 	TSRMLS_FETCH();
 
-	if (!sapi_module.phpinfo_as_text && flag & PHP_CREDITS_FULLPAGE) {
+	if (flag & PHP_CREDITS_FULLPAGE) {
 		php_print_info_htmlhead(TSRMLS_C);
 	}
 
-	if (!sapi_module.phpinfo_as_text) {
-		PUTS("<h1>PHP Credits</h1>\n");
-	} else {
-		PUTS("PHP Credits\n");
-	}
+	PUTS("<h1>PHP Credits</h1>\n");
 
 	if (flag & PHP_CREDITS_GROUP) {
 		/* Group */
@@ -53,11 +48,7 @@ PHPAPI void php_print_credits(int flag)
 	if (flag & PHP_CREDITS_GENERAL) {
 		/* Design & Concept */
 		php_info_print_table_start();
-		if (!sapi_module.phpinfo_as_text) {
-			php_info_print_table_header(1, "Language Design &amp; Concept");
-		} else {
-			php_info_print_table_header(1, "Language Design & Concept");
-		}
+		php_info_print_table_header(1, "Language Design &amp; Concept");
 		php_info_print_table_row(1, "Andi Gutmans, Rasmus Lerdorf, Zeev Suraski");
 		php_info_print_table_end();
 
@@ -106,7 +97,7 @@ PHPAPI void php_print_credits(int flag)
 
 	if (flag & PHP_CREDITS_QA) {
 		php_info_print_table_start();
-		php_info_print_table_header(1, "PHP 4.4 Quality Assurance Team");
+		php_info_print_table_header(1, "PHP 4.3 Quality Assurance Team");
 		php_info_print_table_row(1, "Ilia Alshanetsky, Stefan Esser, Moriyoshi Koizumi, Sebastian Nohn, Derick Rethans, Melvyn Sopacua, Jani Taskinen");
 		php_info_print_table_end();
 	}
@@ -119,7 +110,7 @@ PHPAPI void php_print_credits(int flag)
 		php_info_print_table_end();
 	}
 
-	if (!sapi_module.phpinfo_as_text && flag & PHP_CREDITS_FULLPAGE) {
+	if (flag & PHP_CREDITS_FULLPAGE) {
 		PUTS("</div></body></html>\n");
 	}
 }
