@@ -1,8 +1,8 @@
 /* 
    +----------------------------------------------------------------------+
-   | PHP version 4.0                                                      |
+   | PHP Version 4                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2001 The PHP Group                                |
+   | Copyright (c) 1997-2004 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,9 +18,7 @@
    +----------------------------------------------------------------------+
  */
 
-
-/* $Id: datetime.c,v 1.75.2.1 2001/12/14 10:08:43 sterling Exp $ */
-
+/* $Id: datetime.c,v 1.96.2.16 2004/06/28 14:35:41 derick Exp $ */
 
 #include "php.h"
 #include "zend_operators.h"
@@ -35,37 +33,39 @@
 
 #include "php_parsedate.h"
 
-char *mon_full_names[] =
-{
+char *mon_full_names[] = {
 	"January", "February", "March", "April",
 	"May", "June", "July", "August",
 	"September", "October", "November", "December"
 };
-char *mon_short_names[] =
-{
+
+char *mon_short_names[] = {
 	"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
 };
-char *day_full_names[] =
-{
+
+char *day_full_names[] = {
 	"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"
 };
-char *day_short_names[] =
-{
+
+char *day_short_names[] = {
 	"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"
 };
 
 #if !defined(HAVE_TM_ZONE) && !defined(_TIMEZONE) && !defined(HAVE_DECLARED_TIMEZONE)
+#if defined(NETWARE) && defined(NEW_LIBC)
+#define timezone    _timezone   /* timezone is called '_timezone' in new version of LibC */
+#endif
 extern time_t timezone;
 extern int daylight;
 #endif
 
-static int phpday_tab[2][12] =
-{
+static int phpday_tab[2][12] = {
 	{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31},
 	{31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31}
 };
 
-#define isleap(year) (((year%4) == 0 && (year%100)!=0) || (year%400)==0)
+#define isleap(year) ((((year) % 4) == 0 && ((year) % 100) != 0) || ((year) % 400)==0)
+#define YEAR_BASE 1900
 
 /* {{{ proto int time(void)
    Return current UNIX timestamp */
@@ -81,9 +81,10 @@ void php_mktime(INTERNAL_FUNCTION_PARAMETERS, int gm)
 {
 	pval **arguments[7];
 	struct tm *ta, tmbuf;
-	time_t t;
-	int i, gmadjust, seconds, arg_count = ZEND_NUM_ARGS();
-	int is_dst = -1;
+	time_t t, seconds;
+	int i, gmadjust, arg_count = ZEND_NUM_ARGS();
+	int is_dst = -1, chgsecs = 0;
+	long val;
 
 	if (arg_count > 7 || zend_get_parameters_array_ex(arg_count, arguments) == FAILURE) {
 		WRONG_PARAM_COUNT;
@@ -118,10 +119,18 @@ void php_mktime(INTERNAL_FUNCTION_PARAMETERS, int gm)
 	** Now change date values with supplied parameters.
 	*/
 	switch(arg_count) {
-	case 7:
+	case 7: /* daylight saving time flag */
+#ifdef PHP_WIN32
+		if (daylight > 0) {
+			ta->tm_isdst = is_dst = Z_LVAL_PP(arguments[6]);
+		} else {
+			ta->tm_isdst = is_dst = 0;
+		}
+#else
 		ta->tm_isdst = is_dst = Z_LVAL_PP(arguments[6]);
+#endif
 		/* fall-through */
-	case 6:
+	case 6: /* year */
 		/* special case: 
 		   a zero in year, month and day is considered illegal
 		   as it would be interpreted as 30.11.1999 otherwise
@@ -149,29 +158,86 @@ void php_mktime(INTERNAL_FUNCTION_PARAMETERS, int gm)
 			ta->tm_year = Z_LVAL_PP(arguments[5])
 			  - ((Z_LVAL_PP(arguments[5]) > 1000) ? 1900 : 0);
 		/* fall-through */
-	case 5:
-		ta->tm_mday = Z_LVAL_PP(arguments[4]);
-		/* fall-through */
-	case 4:
-		ta->tm_mon = Z_LVAL_PP(arguments[3]) - 1;
-		/* fall-through */
-	case 3:
-		ta->tm_sec = Z_LVAL_PP(arguments[2]);
-		/* fall-through */
-	case 2:
-		ta->tm_min = Z_LVAL_PP(arguments[1]);
-		/* fall-through */
-	case 1:
-		ta->tm_hour = Z_LVAL_PP(arguments[0]);
-		/* fall-through */
-	case 0:
-		break;
+	case 5: /* day in month (1-based) */
+ 		val = (*arguments[4])->value.lval; 
+		if (val < 1) { 
+			chgsecs += (1-val) * 60*60*24; 
+			val = 1; 			
+		} 
+		ta->tm_mday = val; 
+		/* fall-through */ 
+	case 4: /* month (zero-based) */
+		val = (*arguments[3])->value.lval - 1; 
+		while (val < 0) { 
+			val += 12; ta->tm_year--; 
+		} 
+		ta->tm_mon = val; 
+		/* fall-through */ 
+	case 3: /* second */
+		val = (*arguments[2])->value.lval; 
+		if (val < 1) { 
+			chgsecs += (1-val); val = 1; 
+		} 
+		ta->tm_sec = val; 
+		/* fall-through */ 
+	case 2: /* minute */
+		val = (*arguments[1])->value.lval; 
+		if (val < 1) { 
+			chgsecs += (1-val) * 60; val = 1; 
+		} 
+		ta->tm_min = val; 
+		/* fall-through */ 
+	case 1: /* hour */
+		val = (*arguments[0])->value.lval; 
+		/*
+		   We avoid midnight and a couple of hours after midnight here to work around
+		   various OS-level bugs in mktime and specifically daylight savings time issues
+		   in many mktime implementation.
+		   See bugs #27533 and #27719 for more info.
+		*/
+		if (val < 4) { 
+			chgsecs += (4-val) * 60*60; val = 4; 
+		} 
+		ta->tm_hour = val; 
+		/* fall-through */ 
+	case 0: 
+		break; 
+	} 
+	
+	t = mktime(ta); 
+
+#ifdef PHP_WIN32
+	if (t - chgsecs < 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Windows does not support negative values for this function");
+		RETURN_LONG(-1);
 	}
+#endif
 
-	seconds = mktime(ta);
-	if (is_dst == -1)
-		is_dst = ta->tm_isdst;
+	seconds = t - chgsecs;
 
+	/*
+	   Here we check to see if the chgsecs fuzz factor we applied caused us to
+	   move from dst to non-dst or vice-versa.  If so we adjust accordingly to
+	   avoid being off by an hour on the dst changeover date.
+	*/
+	if (is_dst == -1) {
+		struct tm t1, t2;
+		t1 = *localtime(&t);
+		t2 = *localtime(&seconds);
+
+		if (t1.tm_isdst != t2.tm_isdst) {
+			seconds += (t1.tm_isdst == 1) ? 3600 : -3600;
+			ta = localtime(&seconds);
+		}
+
+		/*
+		   If the user didn't specify whether the timestamp passed in was dst or not
+		   then we fill it in based on the dst setting at the evaluated timestamp
+		   at the current TZ
+		*/
+		is_dst = ta->tm_isdst; 
+	}
+	
 	if (gm) {
 #if HAVE_TM_GMTOFF
 	    /*
@@ -182,10 +248,13 @@ void php_mktime(INTERNAL_FUNCTION_PARAMETERS, int gm)
 #else
 	    /*
 	    ** If correcting for daylight savings time, we set the adjustment to
-		** the value of timezone - 3600 seconds. Otherwise, we need to overcorrect and
-		** set the adjustment to the main timezone + 3600 seconds.
+		** the value of timezone - 3600 seconds.
 	    */
-	    gmadjust = -(is_dst ? timezone - 3600 : timezone + 3600);
+#ifdef __CYGWIN__
+	    gmadjust = -(is_dst ? _timezone - 3600 : _timezone);
+#else
+	    gmadjust = -(is_dst ? timezone - 3600 : timezone);
+#endif
 #endif
 		seconds += gmadjust;
 	}
@@ -212,8 +281,7 @@ PHP_FUNCTION(gmmktime)
 
 /* {{{ php_date
  */
-static void
-php_date(INTERNAL_FUNCTION_PARAMETERS, int gm)
+static void php_date(INTERNAL_FUNCTION_PARAMETERS, int gm)
 {
 	pval **format, **timestamp;
 	time_t the_time;
@@ -238,6 +306,12 @@ php_date(INTERNAL_FUNCTION_PARAMETERS, int gm)
 		}
 		convert_to_long_ex(timestamp);
 		the_time = Z_LVAL_PP(timestamp);
+#ifdef PHP_WIN32
+		if (the_time < 0) {
+			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Windows does not support dates prior to midnight (00:00:00), January 1, 1970");
+			RETURN_FALSE;
+		}
+#endif
 		break;
 	default:
 		WRONG_PARAM_COUNT;
@@ -252,13 +326,25 @@ php_date(INTERNAL_FUNCTION_PARAMETERS, int gm)
 	} else {
 		ta = php_localtime_r(&the_time, &tmbuf);
 #if !HAVE_TM_GMTOFF
+#ifdef __CYGWIN__
+		tzone = _timezone;
+#else
 		tzone = timezone;
-		tname[0] = tzname[0];
+#endif
+		if (tzname[0] != NULL) {
+			tname[0] = tzname[0];
+		} else {
+			tname[0] = "???";
+		}
+
+		if (tzname[1] != NULL) {
+			tname[1] = tzname[1];
+		}
 #endif
 	}
 
 	if (!ta) {			/* that really shouldn't happen... */
-		php_error(E_WARNING, "unexpected error in date()");
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unexpected error");
 		RETURN_FALSE;
 	}
 	for (i = 0; i < Z_STRLEN_PP(format); i++) {
@@ -277,7 +363,11 @@ php_date(INTERNAL_FUNCTION_PARAMETERS, int gm)
 #if HAVE_TM_ZONE
 				size += strlen(ta->tm_zone);
 #elif HAVE_TZNAME
-				size += strlen(tname[0]);
+				if (ta->tm_isdst > 0 ) {
+					size += strlen(tname[1]);
+				} else {
+					size += strlen(tname[0]);
+				}
 #endif
 				break;
 			case 'Z':		/* timezone offset in seconds */
@@ -314,7 +404,7 @@ php_date(INTERNAL_FUNCTION_PARAMETERS, int gm)
 				size += 2;
 				break;
 			case '\\':
-				if(i < Z_STRLEN_PP(format)-1) {
+				if (i < Z_STRLEN_PP(format) - 1) {
 					i++;
 				}
 				size ++;
@@ -334,9 +424,9 @@ php_date(INTERNAL_FUNCTION_PARAMETERS, int gm)
 	for (i = 0; i < Z_STRLEN_PP(format); i++) {
 		switch (Z_STRVAL_PP(format)[i]) {
 			case '\\':
-				if(i < Z_STRLEN_PP(format)-1) {
+				if (i < Z_STRLEN_PP(format) - 1) {
 					char ch[2];
-					ch[0]=Z_STRVAL_PP(format)[i+1];
+					ch[0]=Z_STRVAL_PP(format)[i + 1];
 					ch[1]='\0';
 					strcat(Z_STRVAL_P(return_value), ch);
 					i++;
@@ -353,7 +443,7 @@ php_date(INTERNAL_FUNCTION_PARAMETERS, int gm)
 				strcat(Z_STRVAL_P(return_value), day_full_names[ta->tm_wday]);
 				break;
 			case 'Y':		/* year, numeric, 4 digits */
-				sprintf(tmp_buff, "%d", ta->tm_year + 1900);  /* SAFE */
+				sprintf(tmp_buff, "%d", ta->tm_year + YEAR_BASE);  /* SAFE */
 				strcat(Z_STRVAL_P(return_value), tmp_buff);
 				break;
 			case 'M':		/* month, textual, 3 letters */
@@ -439,7 +529,7 @@ php_date(INTERNAL_FUNCTION_PARAMETERS, int gm)
 				}
 				break;
 			case 't':		/* days in current month */
-				sprintf(tmp_buff, "%2d", phpday_tab[isleap((ta->tm_year+1900))][ta->tm_mon] );
+				sprintf(tmp_buff, "%2d", phpday_tab[isleap((ta->tm_year+YEAR_BASE))][ta->tm_mon] );
 				strcat(Z_STRVAL_P(return_value), tmp_buff);
 				break;
 			case 'w':		/* day of the week, numeric EXTENSION */
@@ -448,9 +538,9 @@ php_date(INTERNAL_FUNCTION_PARAMETERS, int gm)
 				break;
 			case 'O':		/* GMT offset in [+-]HHMM format */
 #if HAVE_TM_GMTOFF				
-				sprintf(tmp_buff, "%c%02d%02d", (ta->tm_gmtoff < 0) ? '-' : '+', abs(ta->tm_gmtoff / 3600), abs( ta->tm_gmtoff % 3600));
+				sprintf(tmp_buff, "%c%02d%02d", (ta->tm_gmtoff < 0) ? '-' : '+', abs(ta->tm_gmtoff / 3600), abs( (ta->tm_gmtoff % 3600) / 60 ));
 #else
-				sprintf(tmp_buff, "%c%02d%02d", ((ta->tm_isdst ? tzone - 3600:tzone)>0)?'-':'+', abs((ta->tm_isdst ? tzone - 3600 : tzone) / 3600), abs((ta->tm_isdst ? tzone - 3600 : tzone) % 3600));
+				sprintf(tmp_buff, "%c%02d%02d", ((ta->tm_isdst ? tzone - 3600:tzone)>0)?'-':'+', abs((ta->tm_isdst ? tzone - 3600 : tzone) / 3600), abs(((ta->tm_isdst ? tzone - 3600 : tzone) % 3600) / 60));
 #endif
 				strcat(Z_STRVAL_P(return_value), tmp_buff);
 				break;
@@ -463,14 +553,14 @@ php_date(INTERNAL_FUNCTION_PARAMETERS, int gm)
 				strcat(Z_STRVAL_P(return_value), tmp_buff);
 				break;
 			case 'L':		/* boolean for leapyear */
-				sprintf(tmp_buff, "%d", (isleap((ta->tm_year+1900)) ? 1 : 0 ) );
+				sprintf(tmp_buff, "%d", (isleap((ta->tm_year+YEAR_BASE)) ? 1 : 0 ) );
 				strcat(Z_STRVAL_P(return_value), tmp_buff);
 				break;
 			case 'T':		/* timezone name */
 #if HAVE_TM_ZONE
 				strcat(Z_STRVAL_P(return_value), ta->tm_zone);
 #elif HAVE_TZNAME
-				strcat(Z_STRVAL_P(return_value), tname[0]);
+				strcat(Z_STRVAL_P(return_value), ta->tm_isdst ? tname[1] : tname[0]);
 #endif
 				break;
 			case 'B':	/* Swatch Beat a.k.a. Internet Time */
@@ -488,43 +578,54 @@ php_date(INTERNAL_FUNCTION_PARAMETERS, int gm)
 				strcat(Z_STRVAL_P(return_value), tmp_buff);
 				break;
 			case 'r':
-#if HAVE_TM_GMTOFF				
-				sprintf(tmp_buff, "%3s, %2d %3s %04d %02d:%02d:%02d %c%02d%02d", 
+#if HAVE_TM_GMTOFF
+				sprintf(tmp_buff, "%3s, %2d %3s %04d %02d:%02d:%02d %c%02d%02d",
 					day_short_names[ta->tm_wday],
 					ta->tm_mday,
 					mon_short_names[ta->tm_mon],
-					ta->tm_year + 1900,
+					ta->tm_year + YEAR_BASE,
 					ta->tm_hour,
 					ta->tm_min,
 					ta->tm_sec,
 					(ta->tm_gmtoff < 0) ? '-' : '+',
 					abs(ta->tm_gmtoff / 3600),
-					abs( ta->tm_gmtoff % 3600)
+					abs( (ta->tm_gmtoff % 3600) / 60 )
 				);
 #else
-				sprintf(tmp_buff, "%3s, %2d %3s %04d %02d:%02d:%02d %c%02d%02d", 
+				sprintf(tmp_buff, "%3s, %2d %3s %04d %02d:%02d:%02d %c%02d%02d",
 					day_short_names[ta->tm_wday],
 					ta->tm_mday,
 					mon_short_names[ta->tm_mon],
-					ta->tm_year + 1900,
+					ta->tm_year + YEAR_BASE,
 					ta->tm_hour,
 					ta->tm_min,
 					ta->tm_sec,
 					((ta->tm_isdst ? tzone - 3600 : tzone) > 0) ? '-' : '+',
 					abs((ta->tm_isdst ? tzone - 3600 : tzone) / 3600),
-					abs((ta->tm_isdst ? tzone - 3600 : tzone) % 3600)
+					abs( ((ta->tm_isdst ? tzone - 3600 : tzone) % 3600) / 60 )
 				);
 #endif
 				strcat(Z_STRVAL_P(return_value), tmp_buff);
 				break;
 			case 'W':		/* ISO-8601 week number of year, weeks starting on Monday */
-				wd = ta->tm_wday==0 ? 7 : ta->tm_wday;
-				yd = ta->tm_yday + 1;
-				fd = (7 + (wd - yd) % 7 ) % 7;
-				wk = ( (yd + fd - 1) / 7 ) + 1;
-				if (fd>3) {
-					wk--;
+				wd = ta->tm_wday == 0 ? 6 : ta->tm_wday - 1; /* weekday */
+				yd = ta->tm_yday + 1;					/* days since January 1st */
+
+				fd = (7 + wd - yd % 7+ 1) % 7;			/* weekday (1st January) */	
+
+				/* week is a last year week (52 or 53) */
+				if ((yd <= 7 - fd) && fd > 3){			
+					wk = (fd == 4 || (fd == 5 && isleap((ta->tm_year + YEAR_BASE - 1)))) ? 53 : 52;
 				}
+				/* week is a next year week (1) */
+				else if (isleap((ta->tm_year+YEAR_BASE)) + 365 - yd < 3 - wd){
+					wk = 1;
+				}
+				/* normal week */
+				else {
+					wk = (yd + 6 - wd + fd) / 7 - (fd > 3);
+				}
+
 				sprintf(tmp_buff, "%d", wk);  /* SAFE */
 				strcat(Z_STRVAL_P(return_value), tmp_buff);
 				break;
@@ -587,11 +688,19 @@ PHP_FUNCTION(localtime)
 			assoc_array = Z_LVAL_PP(assoc_array_arg);
 			break;
 	}
-	ta = php_localtime_r(&timestamp, &tmbuf);
-	if (array_init(return_value) == FAILURE) {
-		php_error(E_ERROR, "Cannot prepare return array from localtime");
+	
+#ifdef PHP_WIN32
+	if (timestamp < 0) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Windows does not support negative values for this function");
+		RETURN_FALSE
+	}
+#endif
+	
+	if (NULL == (ta = php_localtime_r(&timestamp, &tmbuf))) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid local time");
 		RETURN_FALSE;
 	}
+	array_init(return_value);
 
 	if (assoc_array) {
 		add_assoc_long(return_value, "tm_sec",   ta->tm_sec);
@@ -636,13 +745,10 @@ PHP_FUNCTION(getdate)
 
 	ta = php_localtime_r(&timestamp, &tmbuf);
 	if (!ta) {
-		php_error(E_WARNING, "Cannot perform date calculation");
-		return;
-	}
-	if (array_init(return_value) == FAILURE) {
-		php_error(E_ERROR, "Unable to initialize array");
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot perform date calculation");
 		return;
 	}
+	array_init(return_value);
 	add_assoc_long(return_value, "seconds", ta->tm_sec);
 	add_assoc_long(return_value, "minutes", ta->tm_min);
 	add_assoc_long(return_value, "hours", ta->tm_hour);
@@ -659,11 +765,10 @@ PHP_FUNCTION(getdate)
 
 /* {{{ php_std_date
    Return date string in standard format for http headers */
-char *php_std_date(time_t t)
+char *php_std_date(time_t t TSRMLS_DC)
 {
 	struct tm *tm1, tmbuf;
 	char *str;
-	TSRMLS_FETCH();
 
 	tm1 = php_gmtime_r(&t, &tmbuf);
 	str = emalloc(81);
@@ -672,18 +777,18 @@ char *php_std_date(time_t t)
 				day_short_names[tm1->tm_wday],
 				tm1->tm_mday,
 				mon_short_names[tm1->tm_mon],
-				tm1->tm_year+1900,
+				tm1->tm_year + 1900,
 				tm1->tm_hour, tm1->tm_min, tm1->tm_sec);
 	} else {
 		snprintf(str, 80, "%s, %02d-%s-%02d %02d:%02d:%02d GMT",
 				day_short_names[tm1->tm_wday],
 				tm1->tm_mday,
 				mon_short_names[tm1->tm_mon],
-				((tm1->tm_year)%100),
+				((tm1->tm_year) % 100),
 				tm1->tm_hour, tm1->tm_min, tm1->tm_sec);
 	}
 	
-	str[79]=0;
+	str[79] = 0;
 	return (str);
 }
 /* }}} */
@@ -692,37 +797,16 @@ char *php_std_date(time_t t)
    Returns true(1) if it is a valid date in gregorian calendar */
 PHP_FUNCTION(checkdate)
 {
-	pval **month, **day, **year;
-	int m, d, y, res=0;
-	
-	if (ZEND_NUM_ARGS() != 3 ||
-		zend_get_parameters_ex(3, &month, &day, &year) == FAILURE) {
-		WRONG_PARAM_COUNT;
+	long m, d, y;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "lll", &m, &d, &y) == FAILURE) {
+		RETURN_FALSE;
 	}
 
-	if(Z_TYPE_PP(year) == IS_STRING) {
-		res = is_numeric_string(Z_STRVAL_PP(year), Z_STRLEN_PP(year), NULL, NULL, 0);
-		if(res!=IS_LONG && res !=IS_DOUBLE) {
-			RETURN_FALSE;	
-		}
-	}
-	convert_to_long_ex(day);
-	convert_to_long_ex(month);
-	convert_to_long_ex(year);
-	y = Z_LVAL_PP(year);
-	m = Z_LVAL_PP(month);
-	d = Z_LVAL_PP(day);
-
-	if (y < 1 || y > 32767) {
+	if (y < 1 || y > 32767 || m < 1 || m > 12 || d < 1 || d > phpday_tab[isleap(y)][m - 1]) {
 		RETURN_FALSE;
 	}
-	if (m < 1 || m > 12) {
-		RETURN_FALSE;
-	}
-	if (d < 1 || d > phpday_tab[isleap(y)][m - 1]) {
-		RETURN_FALSE;
-	}
-	RETURN_TRUE;				/* True : This month, day, year arguments are valid */
+	RETURN_TRUE;	/* True : This month, day, year arguments are valid */
 }
 /* }}} */
 
@@ -761,9 +845,11 @@ void _php_strftime(INTERNAL_FUNCTION_PARAMETERS, int gm)
 	if (Z_STRLEN_PP(format_arg)==0) {
 		RETURN_FALSE;
 	}
+#ifdef PHP_WIN32
 	if (timestamp < 0) {
 		RETURN_FALSE;
 	}
+#endif
 	format = Z_STRVAL_PP(format_arg);
 	if (gm) {
 		ta = php_gmtime_r(&timestamp, &tmbuf);
@@ -775,11 +861,13 @@ void _php_strftime(INTERNAL_FUNCTION_PARAMETERS, int gm)
 	while ((real_len=strftime(buf, buf_len, format, ta))==buf_len || real_len==0) {
 		buf_len *= 2;
 		buf = (char *) erealloc(buf, buf_len);
-		if(!--max_reallocs) break;
+		if (!--max_reallocs) {
+			break;
+		}
 	}
 	
-	if(real_len && real_len != buf_len) {
-		buf = (char *) erealloc(buf, real_len+1);
+	if (real_len && real_len != buf_len) {
+		buf = (char *) erealloc(buf, real_len + 1);
 		RETURN_STRINGL(buf, real_len, 0);
 	}
 	efree(buf);
@@ -821,7 +909,7 @@ PHP_FUNCTION(strtotime)
 
 	convert_to_string_ex(z_time);
 	if (Z_STRLEN_PP(z_time) == 0)
-		php_error (E_NOTICE, "strtotime() called with empty time parameter");
+		php_error_docref(NULL TSRMLS_CC, E_NOTICE, "Called with empty time parameter");
 	if (argc == 2) {
 		convert_to_long_ex(z_now);
 		now = Z_LVAL_PP(z_now);
