@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP version 4.0                                                      |
+   | PHP Version 4                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2001 The PHP Group                                |
+   | Copyright (c) 1997-2003 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 2.02 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -18,7 +18,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: dl.c,v 1.59.2.1 2001/10/23 01:19:59 zeev Exp $ */
+/* $Id: dl.c,v 1.79.2.3 2003/01/29 15:40:24 edink Exp $ */
 
 #include "php.h"
 #include "dl.h"
@@ -26,15 +26,11 @@
 #include "ext/standard/info.h"
 #include "SAPI.h"
 
-#ifndef PHP_WIN32
-#include "build-defs.h"
-#endif
-
-#ifdef HAVE_LIBDL
+#if defined(HAVE_LIBDL) || HAVE_MACH_O_DYLD_H
 #include <stdlib.h>
 #include <stdio.h>
 
-#if HAVE_STRING_H
+#ifdef HAVE_STRING_H
 #include <string.h>
 #else
 #include <strings.h>
@@ -43,12 +39,19 @@
 #include "win32/param.h"
 #include "win32/winutil.h"
 #define GET_DL_ERROR()	php_win_err()
+#elif defined(NETWARE)
+#ifdef NEW_LIBC
+#include <sys/param.h>
+#else
+#include "netware/param.h"
+#endif
+#define GET_DL_ERROR()	dlerror()
 #else
 #include <sys/param.h>
-#define GET_DL_ERROR()	dlerror()
+#define GET_DL_ERROR()	DL_ERROR()
 #endif
 
-#endif
+#endif /* defined(HAVE_LIBDL) || HAVE_MACH_O_DYLD_H */
 
 
 /* {{{ proto int dl(string extension_filename)
@@ -58,8 +61,11 @@ PHP_FUNCTION(dl)
 	pval **file;
 
 #ifdef ZTS
-	if (strcmp(sapi_module.name, "cgi")!=0) {
-		php_error(E_ERROR, "dl() is not supported in multithreaded Web servers - use extension statements in your php.ini");
+	if ((strncmp(sapi_module.name, "cgi", 3)!=0) && 
+		(strcmp(sapi_module.name, "cli")!=0) &&
+		(strncmp(sapi_module.name, "embed", 5)!=0)) {
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Not supported in multithreaded Web servers - use extension statements in your php.ini");
+		RETURN_FALSE;
 	}
 #endif
 
@@ -71,9 +77,9 @@ PHP_FUNCTION(dl)
 	convert_to_string_ex(file);
 
 	if (!PG(enable_dl)) {
-		php_error(E_ERROR, "Dynamically loaded extentions aren't enabled.");
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Dynamically loaded extentions aren't enabled");
 	} else if (PG(safe_mode)) {
-		php_error(E_ERROR, "Dynamically loaded extensions aren't allowed when running in SAFE MODE.");
+		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Dynamically loaded extensions aren't allowed when running in Safe Mode");
 	} else {
 		php_dl(*file, MODULE_TEMPORARY, return_value TSRMLS_CC);
 		EG(full_tables_cleanup) = 1;
@@ -83,7 +89,7 @@ PHP_FUNCTION(dl)
 /* }}} */
 
 
-#ifdef HAVE_LIBDL
+#if defined(HAVE_LIBDL) || HAVE_MACH_O_DYLD_H
 
 #ifdef ZTS
 #define USING_ZTS 1
@@ -120,28 +126,28 @@ void php_dl(pval *file, int type, pval *return_value TSRMLS_DC)
 	if (extension_dir && extension_dir[0]){
 		int extension_dir_len = strlen(extension_dir);
 
-		libpath = emalloc(extension_dir_len+file->value.str.len+2);
+		libpath = emalloc(extension_dir_len+Z_STRLEN_P(file)+2);
 
 		if (IS_SLASH(extension_dir[extension_dir_len-1])) {
-			sprintf(libpath, "%s%s", extension_dir, file->value.str.val); /* SAFE */
+			sprintf(libpath, "%s%s", extension_dir, Z_STRVAL_P(file)); /* SAFE */
 		} else {
-			sprintf(libpath, "%s/%s", extension_dir, file->value.str.val); /* SAFE */
+			sprintf(libpath, "%s%c%s", extension_dir, DEFAULT_SLASH, Z_STRVAL_P(file)); /* SAFE */
 		}
 	} else {
-		libpath = estrndup(file->value.str.val, file->value.str.len);
+		libpath = estrndup(Z_STRVAL_P(file), Z_STRLEN_P(file));
 	}
 
 	/* load dynamic symbol */
 	handle = DL_LOAD(libpath);
 	if (!handle) {
-		php_error(error_type, "Unable to load dynamic library '%s' - %s", libpath, GET_DL_ERROR());
+		php_error_docref(NULL TSRMLS_CC, error_type, "Unable to load dynamic library '%s' - %s", libpath, GET_DL_ERROR());
 		efree(libpath);
 		RETURN_FALSE;
 	}
 
 	efree(libpath);
 
-	
+#ifndef NETWARE
 	get_module = (zend_module_entry *(*)(void)) DL_FETCH_SYMBOL(handle, "get_module");
 
 	/*
@@ -152,30 +158,85 @@ void php_dl(pval *file, int type, pval *return_value TSRMLS_DC)
 
 	if (!get_module)
 		get_module = (zend_module_entry *(*)(void)) DL_FETCH_SYMBOL(handle, "_get_module");
+#else
+	/* NetWare doesn't support two NLMs exporting same symbol */
+	{
+		char symbol_name[64] = "\0";
+		int module_name_length = Z_STRLEN_P(file) - 4;  /* '.nlm' is 4 characters; knock it off */
+
+		/* Take the module name (e.g.: 'php_ldap') and append '@get_module' to it */
+		strncpy(symbol_name, Z_STRVAL_P(file), module_name_length);
+		symbol_name[module_name_length] = '\0';
+		strcat(symbol_name, "@");
+		strcat(symbol_name, "get_module");
+
+		get_module = (zend_module_entry *(*)(void)) DL_FETCH_SYMBOL(handle, symbol_name);
+	}
+	/* NetWare doesn't prepend '_' to symbol names; so the corresponding portion of code is also
+	   not required for NetWare */
+#endif
 
 	if (!get_module) {
 		DL_UNLOAD(handle);
-		php_error(error_type, "Invalid library (maybe not a PHP library) '%s' ", file->value.str.val);
+		php_error_docref(NULL TSRMLS_CC, error_type, "Invalid library (maybe not a PHP library) '%s' ", Z_STRVAL_P(file));
 		RETURN_FALSE;
 	}
 	module_entry = get_module();
 	if ((module_entry->zend_debug != ZEND_DEBUG) || (module_entry->zts != USING_ZTS)
 		|| (module_entry->zend_api != ZEND_MODULE_API_NO)) {
-		php_error(error_type,
-					"%s: Unable to initialize module\n"
-					"Module compiled with debug=%d, thread-safety=%d module API=%d\n"
-					"PHP compiled with debug=%d, thread-safety=%d module API=%d\n"
-					"These options need to match\n",
-					module_entry->name, module_entry->zend_debug, module_entry->zts, module_entry->zend_api,
-					ZEND_DEBUG, USING_ZTS, ZEND_MODULE_API_NO);
-		DL_UNLOAD(handle);
-		RETURN_FALSE;
+		/* Check for pre-4.1.0 module which has a slightly different module_entry structure :( */
+			struct pre_4_1_0_module_entry {
+				  char *name;
+				  zend_function_entry *functions;
+				  int (*module_startup_func)(INIT_FUNC_ARGS);
+				  int (*module_shutdown_func)(SHUTDOWN_FUNC_ARGS);
+				  int (*request_startup_func)(INIT_FUNC_ARGS);
+				  int (*request_shutdown_func)(SHUTDOWN_FUNC_ARGS);
+				  void (*info_func)(ZEND_MODULE_INFO_FUNC_ARGS);
+				  int (*global_startup_func)(void);
+				  int (*global_shutdown_func)(void);
+				  int globals_id;
+				  int module_started;
+				  unsigned char type;
+				  void *handle;
+				  int module_number;
+				  unsigned char zend_debug;
+				  unsigned char zts;
+				  unsigned int zend_api;
+			};
+
+			char *name;
+			int zend_api;
+			unsigned char zend_debug, zts;
+
+			if((  ((struct pre_4_1_0_module_entry *)module_entry)->zend_api > 20000000)
+			   &&(((struct pre_4_1_0_module_entry *)module_entry)->zend_api < 20010901)) {
+				name       = ((struct pre_4_1_0_module_entry *)module_entry)->name;
+				zend_api   = ((struct pre_4_1_0_module_entry *)module_entry)->zend_api;
+				zend_debug = ((struct pre_4_1_0_module_entry *)module_entry)->zend_debug;
+				zts        = ((struct pre_4_1_0_module_entry *)module_entry)->zts; 
+			} else {			
+				name       = module_entry->name;
+				zend_api   = module_entry->zend_api;
+				zend_debug = module_entry->zend_debug;
+				zts        = module_entry->zts; 
+			}
+
+			php_error_docref(NULL TSRMLS_CC, error_type,
+					  "%s: Unable to initialize module\n"
+					  "Module compiled with module API=%d, debug=%d, thread-safety=%d\n"
+					  "PHP    compiled with module API=%d, debug=%d, thread-safety=%d\n"
+					  "These options need to match\n",
+					  name, zend_api, zend_debug, zts,
+					  ZEND_MODULE_API_NO, ZEND_DEBUG, USING_ZTS);
+			DL_UNLOAD(handle);
+			RETURN_FALSE;
 	}
-	module_entry->type = type;
+	Z_TYPE_P(module_entry) = type;
 	module_entry->module_number = zend_next_free_module();
 	if (module_entry->module_startup_func) {
 		if (module_entry->module_startup_func(type, module_entry->module_number TSRMLS_CC)==FAILURE) {
-			php_error(error_type, "%s:  Unable to initialize module", module_entry->name);
+			php_error_docref(NULL TSRMLS_CC, error_type, "Unable to initialize module '%s'", module_entry->name);
 			DL_UNLOAD(handle);
 			RETURN_FALSE;
 		}
@@ -184,7 +245,7 @@ void php_dl(pval *file, int type, pval *return_value TSRMLS_DC)
 
 	if ((type == MODULE_TEMPORARY) && module_entry->request_startup_func) {
 		if (module_entry->request_startup_func(type, module_entry->module_number TSRMLS_CC)) {
-			php_error(error_type, "%s:  Unable to initialize module", module_entry->name);
+			php_error_docref(NULL TSRMLS_CC, error_type, "Unable to initialize module '%s'", module_entry->name);
 			DL_UNLOAD(handle);
 			RETURN_FALSE;
 		}
@@ -192,7 +253,7 @@ void php_dl(pval *file, int type, pval *return_value TSRMLS_DC)
 	
 	/* update the .request_started property... */
 	if (zend_hash_find(&module_registry, module_entry->name, strlen(module_entry->name)+1, (void **) &tmp)==FAILURE) {
-		php_error(error_type, "%s:  Loaded module got lost", module_entry->name);
+		php_error_docref(NULL TSRMLS_CC, error_type, "Loaded module '%s' got lost", module_entry->name);
 		RETURN_FALSE;
 	}
 	tmp->handle = handle;
@@ -208,15 +269,15 @@ PHP_MINFO_FUNCTION(dl)
 
 #else
 
-void php_dl(pval *file, int type, pval *return_value)
+void php_dl(pval *file, int type, pval *return_value TSRMLS_DC)
 {
-	php_error(E_WARNING, "Cannot dynamically load %s - dynamic modules are not supported", file->value.str.val);
+	php_error_docref(NULL TSRMLS_CC, E_WARNING, "Cannot dynamically load %s - dynamic modules are not supported", Z_STRVAL_P(file));
 	RETURN_FALSE;
 }
 
 PHP_MINFO_FUNCTION(dl)
 {
-	PUTS("Dynamic Library support not available<br>.\n");
+	PUTS("Dynamic Library support not available<br />.\n");
 }
 
 #endif
@@ -226,6 +287,6 @@ PHP_MINFO_FUNCTION(dl)
  * tab-width: 4
  * c-basic-offset: 4
  * End:
- * vim600: sw=4 ts=4 tw=78 fdm=marker
- * vim<600: sw=4 ts=4 tw=78
+ * vim600: sw=4 ts=4 fdm=marker
+ * vim<600: sw=4 ts=4
  */
