@@ -18,7 +18,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: info.c,v 1.218.2.14 2003/09/22 04:18:39 iliaa Exp $ */
+/* $Id: info.c,v 1.218.2.16 2004/06/09 15:10:19 iliaa Exp $ */
 
 #include "php.h"
 #include "php_ini.h"
@@ -411,7 +411,7 @@ PHPAPI void php_print_info(int flag TSRMLS_DC)
 				PUTS(SG(request_info).request_uri);
 			}
 			if ((ta->tm_mon==3) && (ta->tm_mday==1)) {
-				PUTS("?="PHP_EGG_LOGO_GUID"\" alt=\"Thies!\" /></a>");
+				PUTS("?="PHP_EGG_LOGO_GUID"\" alt=\"Nadia!\" /></a>");
 			} else {
 				PUTS("?="PHP_LOGO_GUID"\" alt=\"PHP Logo\" /></a>");
 			}
@@ -974,7 +974,7 @@ PHP_FUNCTION(php_uname)
 /* {{{ proto string php_ini_scanned_files(void)
    Return comma-separated string of .ini files parsed from the additional ini dir */
 PHP_FUNCTION(php_ini_scanned_files) {
-	if(strlen(PHP_CONFIG_FILE_SCAN_DIR)) {
+	if(strlen(PHP_CONFIG_FILE_SCAN_DIR) && php_ini_scanned_files) {
 		RETURN_STRING(php_ini_scanned_files,1);
 	} else RETURN_FALSE;
 }
