@@ -3,7 +3,7 @@
 // +----------------------------------------------------------------------+
 // | PHP Version 4                                                        |
 // +----------------------------------------------------------------------+
-// | Copyright (c) 1997-2003 The PHP Group                                |
+// | Copyright (c) 1997-2004 The PHP Group                                |
 // +----------------------------------------------------------------------+
 // | This source file is subject to version 3.0 of the PHP license,       |
 // | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
 // |                                                                      |
 // +----------------------------------------------------------------------+
 //
-// $Id: Remote.php,v 1.22.2.13 2003/10/20 15:51:45 cox Exp $
+// $Id: Remote.php,v 1.22.2.17 2004/04/03 15:56:11 cellog Exp $
 
 require_once 'PEAR/Command/Common.php';
 require_once 'PEAR/Common.php';
@@ -250,14 +250,14 @@ parameter.
 
         $r = new PEAR_Remote($this->config);
         $reg = new PEAR_Registry($this->config->get('php_dir'));
-        $available = $r->call('package.listAll', true);
+        $available = $r->call('package.listAll', true, false);
         if (PEAR::isError($available)) {
             return $this->raiseError($available);
         }
         $data = array(
             'caption' => 'Matched packages:',
             'border' => true,
-            'headline' => array('Package', 'Latest', 'Local'),
+            'headline' => array('Package', 'Stable/(Latest)', 'Local'),
             );
 
         foreach ($available as $name => $info) {
@@ -274,16 +274,23 @@ parameter.
             if (isset($params[$name]))
                 $desc .= "\n\n".$info['description'];
 
+            $unstable = '';
+            if ($info['unstable']) {
+                $unstable = '/(' . $info['unstable'] . $info['state'] . ')';
+            }
+            if (!isset($info['stable']) || !$info['stable']) {
+                $info['stable'] = 'none';
+            }
             $data['data'][$info['category']][] = array(
                 $name,
-                $info['stable'],
+                $info['stable'] . $unstable,
                 $installed['version'],
                 $desc,
                 );
         }
         if (!isset($data['data'])) {
             return $this->raiseError('no packages found');
-        };
+        }
         $this->ui->outputData($data, $command);
         return true;
     }
@@ -352,7 +359,7 @@ parameter.
             'border' => 1,
             'headline' => array('Package', 'Local', 'Remote', 'Size'),
             );
-        foreach ($latest as $pkg => $info) {
+        foreach ((array)$latest as $pkg => $info) {
             $package = strtolower($pkg);
             if (!isset($inst[$package])) {
                 // skip packages we don't have installed
