@@ -20,7 +20,7 @@
    +----------------------------------------------------------------------+
 */
 
-/* $Id: cgi_main.c,v 1.190.2.68.2.1 2005/10/06 20:39:26 johannes Exp $ */
+/* $Id: cgi_main.c,v 1.190.2.62 2004/07/14 22:38:18 edink Exp $ */
 
 #include "php.h"
 #include "php_globals.h"
@@ -481,20 +481,6 @@ static void sapi_cgi_register_variables(zval *track_vars_array TSRMLS_DC)
 
 static void sapi_cgi_log_message(char *message)
 {
-#if PHP_FASTCGI
-	long logging = 1;
-	TSRMLS_FETCH();
-
-	if (cfg_get_long("fastcgi.logging", &logging) == FAILURE) {
-		logging = 1;
-	}
-                                                                                                        
-	if (!FCGX_IsCGI() && logging) {
-		FCGX_Request *request = (FCGX_Request *)SG(server_context);
-		FCGX_FPrintF( request->err, "%s\n", message );
-		/* ignore return code */
-	} else
-#endif /* PHP_FASTCGI */
 	fprintf(stderr, "%s\n", message);
 }
 
@@ -1003,7 +989,7 @@ int main(int argc, char *argv[])
 	int fcgi_fd = 0;
 	FCGX_Request request;
 #ifdef PHP_WIN32
-	long impersonate = 0;
+	int impersonate = 0;
 #else
     int status = 0;
 #endif
@@ -1395,7 +1381,7 @@ consult the installation file that came with this distribution, or visit \n\
 						}
 						php_print_info(0xFFFFFFFF TSRMLS_CC);
 						php_end_ob_buffers(1 TSRMLS_CC);
-						exit(0);
+						exit(1);
 						break;
 
   				case 'l': /* syntax check mode */
@@ -1413,7 +1399,7 @@ consult the installation file that came with this distribution, or visit \n\
 					print_extensions(TSRMLS_C);
 					php_printf("\n");
 					php_end_ob_buffers(1 TSRMLS_CC);
-					exit(0);
+					exit(1);
 					break;
 
 #if 0 /* not yet operational, see also below ... */
@@ -1446,7 +1432,7 @@ consult the installation file that came with this distribution, or visit \n\
 						php_printf("PHP %s (%s) (built: %s %s)\nCopyright (c) 1997-2004 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__, get_zend_version());
 #endif
 						php_end_ob_buffers(1 TSRMLS_CC);
-						exit(0);
+						exit(1);
 						break;
 
   				case 'w': 
